@@ -33,6 +33,7 @@ module Database.Persist.Class
     , PersistField (..)
     -- * PersistConfig
     , PersistConfig (..)
+    , entityValues
 
     -- * Lifting
     , HasPersistBackend (..)
@@ -41,6 +42,7 @@ module Database.Persist.Class
     -- * JSON utilities
     , keyValueEntityToJSON, keyValueEntityFromJSON
     , entityIdToJSON, entityIdFromJSON
+    , toPersistValueJSON, fromPersistValueJSON
     ) where
 
 import Database.Persist.Class.DeleteCascade
