@@ -1,7 +1,9 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE ExistentialQuantification #-}
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RankNTypes #-}
+{-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE FlexibleContexts, StandaloneDeriving, UndecidableInstances #-}
 module Database.Persist.Class.PersistEntity
     ( PersistEntity (..)
@@ -12,19 +14,31 @@ module Database.Persist.Class.PersistEntity
     , BackendSpecificFilter
     , Entity (..)
 
+    , entityValues
     , keyValueEntityToJSON, keyValueEntityFromJSON
     , entityIdToJSON, entityIdFromJSON
+      -- * PersistField based on other typeclasses
+    , toPersistValueJSON, fromPersistValueJSON
+    , toPersistValueEnum, fromPersistValueEnum
     ) where
 
 import Database.Persist.Types.Base
 import Database.Persist.Class.PersistField
 import Data.Text (Text)
 import qualified Data.Text as T
-import Data.Aeson (ToJSON (..), FromJSON (..), object, (.:), (.=), Value (Object))
-import Data.Aeson.Types (Parser)
+import qualified Data.Text.Encoding as TE
+import qualified Data.Text.Lazy as LT
+import qualified Data.Text.Lazy.Builder as TB
+import Data.Aeson (ToJSON (..), FromJSON (..), fromJSON, object, (.:), (.=), Value (Object))
+import qualified Data.Aeson.Parser as AP
+import Data.Aeson.Types (Parser,Result(Error,Success))
+import Data.Aeson.Encode (encodeToTextBuilder)
+import Data.Attoparsec.ByteString (parseOnly)
 import Control.Applicative ((<$>), (<*>))
 import Data.Monoid (mappend)
 import qualified Data.HashMap.Strict as HM
+import Data.Typeable (Typeable)
+import Data.Maybe (isJust)
 
 -- | Persistent serialized Haskell records to the database.
 -- A Database 'Entity' (A row in SQL, a document in MongoDB, etc)
@@ -157,6 +171,20 @@ deriving instance (PersistEntity record, Eq (Key record), Eq record) => Eq (Enti
 deriving instance (PersistEntity record, Ord (Key record), Ord record) => Ord (Entity record)
 deriving instance (PersistEntity record, Show (Key record), Show record) => Show (Entity record)
 deriving instance (PersistEntity record, Read (Key record), Read record) => Read (Entity record)
+#if MIN_VERSION_base(4,7,0)
+deriving instance Typeable Entity
+#endif
+
+entityValues :: PersistEntity record => Entity record -> [PersistValue]
+entityValues (Entity k record) =
+  if isJust (entityPrimary ent)
+    then
+      -- TODO: check against the key
+      map toPersistValue (toPersistFields record)
+    else
+      keyToValues k ++ map toPersistValue (toPersistFields record)
+  where
+    ent = entityDef $ Just record
 
 -- | Predefined @toJSON@. The resulting JSON looks like
 -- @{\"key\": 1, \"value\": {\"name\": ...}}@.
@@ -243,3 +271,88 @@ errMsg = mappend "PersistField entity fromPersistValue: "
 -- so lets use MongoDB conventions
 idField :: Text
 idField = "_id"
+
+-- | Convenience function for getting a free 'PersistField' instance
+--   from a type with JSON instances.
+--
+--
+-- Example usage in combination with`fromPersistValueJSON`:
+--
+-- @
+-- instance PersistField MyData where
+--   fromPersistValue = fromPersistValueJSON
+--   toPersistValue = toPersistValueJSON
+-- @
+--
+toPersistValueJSON :: ToJSON a => a -> PersistValue
+toPersistValueJSON = PersistText . LT.toStrict . TB.toLazyText . encodeToTextBuilder . toJSON
+
+-- | Convenience function for getting a free 'PersistField' instance
+--   from a type with JSON instances. The JSON parser used will accept
+--   JSON values other that object and arrays. So, if your instance
+--   serializes the data to a JSON string, this will still work.
+--
+--
+-- Example usage in combination with`toPersistValueJSON`:
+--
+-- @
+-- instance PersistField MyData where
+--   fromPersistValue = fromPersistValueJSON
+--   toPersistValue = toPersistValueJSON
+-- @
+--
+fromPersistValueJSON :: FromJSON a => PersistValue -> Either Text a
+fromPersistValueJSON z = case z of
+  PersistByteString bs -> mapLeft (T.append "Could not parse the JSON (was a PersistByteString): ")
+                        $ parseGo bs
+  PersistText t -> mapLeft (T.append "Could not parse the JSON (was PersistText): ")
+                 $ parseGo (TE.encodeUtf8 t)
+  a -> Left $ T.append "Expected PersistByteString, received: " (T.pack (show a))
+  where parseGo bs = mapLeft T.pack $ case parseOnly AP.value bs of
+          Left err -> Left err
+          Right v -> case fromJSON v of
+            Error err -> Left err
+            Success a -> Right a
+        mapLeft _ (Right a) = Right a
+        mapLeft f (Left b)  = Left (f b)
+
+-- | Convenience function for getting a free 'PersistField' instance
+-- from a type with an 'Enum' instance. The function 'derivePersistField'
+-- from the persistent-template package should generally be preferred.
+-- However, if you want to ensure that an @ORDER BY@ clause that uses
+-- your field will order rows by the data constructor order, this is
+-- a better choice.
+--
+-- Example usage in combination with `fromPersistValueEnum`:
+--
+-- @
+-- data SeverityLevel = Low | Medium | Critical | High
+--   deriving (Enum, Bounded)
+-- instance PersistField SeverityLevel where
+--   fromPersistValue = fromPersistValueEnum
+--   toPersistValue = toPersistValueEnum
+-- @
+toPersistValueEnum :: Enum a => a -> PersistValue
+toPersistValueEnum = toPersistValue . fromEnum
+
+-- | Convenience function for getting a free 'PersistField' instance
+-- from a type with an 'Enum' instance. This function also requires
+-- a `Bounded` instance to improve the reporting of errors.
+--
+-- Example usage in combination with `toPersistValueEnum`:
+--
+-- @
+-- data SeverityLevel = Low | Medium | Critical | High
+--   deriving (Enum, Bounded)
+-- instance PersistField SeverityLevel where
+--   fromPersistValue = fromPersistValueEnum
+--   toPersistValue = toPersistValueEnum
+-- @
+fromPersistValueEnum :: (Enum a, Bounded a) => PersistValue -> Either Text a
+fromPersistValueEnum v = fromPersistValue v >>= go
+  where go i = let res = toEnum i in
+               if i >= fromEnum (asTypeOf minBound res) && i <= fromEnum (asTypeOf maxBound res)
+                 then Right res
+                 else Left ("The number " `mappend` T.pack (show i) `mappend` " was out of the "
+                  `mappend` "allowed bounds for an enum type")
+
