@@ -1,7 +1,5 @@
-{-# LANGUAGE CPP #-}
 {-# LANGUAGE TypeFamilies, FlexibleContexts #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
-{-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE FunctionalDependencies #-}
 module Database.Persist.Class.PersistStore
     ( HasPersistBackend (..)
@@ -14,21 +12,11 @@ module Database.Persist.Class.PersistStore
     , ToBackendKey(..)
     ) where
 
-import qualified Prelude
-import Prelude hiding ((++), show)
-
 import qualified Data.Text as T
-
-import Control.Monad.Trans.Error (Error (..))
 import Control.Monad.IO.Class (MonadIO, liftIO)
 import Control.Exception.Lifted (throwIO)
-
-import Data.Conduit.Internal (Pipe, ConduitM)
-
-import Control.Monad.Trans.Reader   ( ReaderT  )
+import Control.Monad.Trans.Reader (ReaderT)
 import Control.Monad.Reader (MonadReader (ask), runReaderT)
-
-
 import Database.Persist.Class.PersistEntity
 import Database.Persist.Class.PersistField
 import Database.Persist.Types
@@ -80,31 +68,36 @@ class
             => val -> ReaderT backend m ()
     insert_ val = insert val >> return ()
 
-    -- | Create multiple records in the database.
+    -- | Create multiple records in the database and return their 'Key's.
     --
-    -- If you don't need the inserted @Key@s, use 'insertMany_'
+    -- If you don't need the inserted 'Key's, use 'insertMany_'.
     --
-    -- Some backends use the slow default implementation of
-    -- @mapM insert@
+    -- The MongoDB and PostgreSQL backends insert all records and
+    -- retrieve their keys in one database query.
+    --
+    -- The SQLite and MySQL backends use the slow, default implementation of
+    -- @mapM insert@.
     insertMany :: (MonadIO m, backend ~ PersistEntityBackend val, PersistEntity val)
                => [val] -> ReaderT backend m [Key val]
     insertMany = mapM insert
 
-    -- | Same as 'insertMany', but doesn't return any @Key@s.
+    -- | Same as 'insertMany', but doesn't return any 'Key's.
     --
-    -- SQL backends currently use an efficient implementation for this
-    -- unlike 'insertMany'.
+    -- The MongoDB, PostgreSQL, SQLite and MySQL backends insert all records in
+    -- one database query.
     insertMany_ :: (MonadIO m, backend ~ PersistEntityBackend val, PersistEntity val)
                 => [val] -> ReaderT backend m ()
     insertMany_ x = insertMany x >> return ()
 
-    -- | Same as 'insertMany_', but takes an 'Entity' instead of just a record
+    -- | Same as 'insertMany_', but takes an 'Entity' instead of just a record.
     --
     -- Useful when migrating data from one entity to another
-    -- and want to preserve ids
+    -- and want to preserve ids.
     --
-    -- Some backends use the slow default implementation of
-    -- @mapM insertKey@
+    -- The MongoDB backend inserts all the entities in one database query.
+    --
+    -- The SQL backends use the slow, default implementation of
+    -- @mapM_ insertKey@.
     insertEntityMany :: (MonadIO m, backend ~ PersistEntityBackend val, PersistEntity val)
                      => [Entity val] -> ReaderT backend m ()
     insertEntityMany = mapM_ (\(Entity k record) -> insertKey k record)
@@ -144,7 +137,7 @@ class
               => Key val -> [Update val] -> ReaderT backend m val
     updateGet key ups = do
         update key ups
-        get key >>= maybe (liftIO $ throwIO $ KeyNotFound $ Prelude.show key) return
+        get key >>= maybe (liftIO $ throwIO $ KeyNotFound $ show key) return
 
 
 -- | Same as get, but for a non-null (not Maybe) foreign key
@@ -156,7 +149,7 @@ getJust :: ( PersistStore backend
            , MonadIO m
            ) => Key val -> ReaderT backend m val
 getJust key = get key >>= maybe
-  (liftIO $ throwIO $ PersistForeignConstraintUnmet $ T.pack $ Prelude.show key)
+  (liftIO $ throwIO $ PersistForeignConstraintUnmet $ T.pack $ show key)
   return
 
 -- | curry this to make a convenience function that loads an associated model
