@@ -34,6 +34,10 @@ import Text.Blaze.Html (Html)
 import Data.Bits (bitSize)
 import qualified Data.Vector as V
 
+#if MIN_VERSION_base(4,8,0)
+import Numeric.Natural (Natural)
+#endif
+
 -- | Class for data types that may be retrived from a 'rawSql'
 -- query.
 class RawSql a where
@@ -274,6 +278,11 @@ instance (HasResolution a) => PersistFieldSql (Fixed a) where
 instance PersistFieldSql Rational where
     sqlType _ = SqlNumeric 32 20   --  need to make this field big enough to handle Rational to Mumber string conversion for ODBC
 
+#if MIN_VERSION_base(4,8,0)
+instance PersistFieldSql Natural where
+  sqlType _ = SqlInt64
+#endif
+
 -- An embedded Entity
 instance (PersistField record, PersistEntity record) => PersistFieldSql (Entity record) where
     sqlType _ = SqlString
