@@ -3,7 +3,7 @@
 -- | Intended for creating new backends.
 module Database.Persist.Sql.Internal
     ( mkColumns
-    , convertKey
+    , defaultAttribute
     ) where
 
 import Database.Persist.Types
@@ -11,12 +11,17 @@ import Database.Persist.Quasi
 import Data.Char (isSpace)
 import Data.Text (Text)
 import qualified Data.Text as T
-import Data.Monoid (Monoid, mappend, mconcat)
-import Data.Maybe (mapMaybe, listToMaybe)
+import Data.Monoid (mappend, mconcat)
 import Database.Persist.Sql.Types
 
+defaultAttribute :: [Attr] -> Maybe Text
+defaultAttribute [] = Nothing
+defaultAttribute (a:as)
+    | Just d <- T.stripPrefix "default=" a = Just d
+    | otherwise = defaultAttribute as
+
 -- | Create the list of columns for the given entity.
-mkColumns :: [EntityDef a] -> EntityDef SqlType -> ([Column], [UniqueDef], [ForeignDef])
+mkColumns :: [EntityDef] -> EntityDef -> ([Column], [UniqueDef], [ForeignDef])
 mkColumns allDefs t =
     (cols, entityUniques t, entityForeigns t)
   where
@@ -26,25 +31,16 @@ mkColumns allDefs t =
     tn :: DBName
     tn = entityDB t
 
-    go :: FieldDef SqlType -> Column
+    go :: FieldDef -> Column
     go fd =
         Column
             (fieldDB fd)
             (nullable (fieldAttrs fd) /= NotNullable || entitySum t)
-            (maybe
-                (fieldSqlType fd)
-                SqlOther
-                (listToMaybe $ mapMaybe (T.stripPrefix "sqltype=") $ fieldAttrs fd))
-            (def $ fieldAttrs fd)
+            (fieldSqlType fd)
+            (defaultAttribute $ fieldAttrs fd)
             Nothing
             (maxLen $ fieldAttrs fd)
-            (ref (fieldDB fd) (fieldType fd) (fieldAttrs fd))
-
-    def :: [Attr] -> Maybe Text
-    def [] = Nothing
-    def (a:as)
-        | Just d <- T.stripPrefix "default=" a = Just d
-        | otherwise = def as
+            (ref (fieldDB fd) (fieldReference fd) (fieldAttrs fd))
 
     maxLen :: [Attr] -> Maybe Integer
     maxLen [] = Nothing
@@ -57,12 +53,12 @@ mkColumns allDefs t =
         | otherwise = maxLen as
 
     ref :: DBName
-        -> FieldType
+        -> ReferenceDef
         -> [Attr]
         -> Maybe (DBName, DBName) -- table name, constraint name
-    ref c ft []
-        | Just f <- stripId ft =
-            Just (resolveTableName allDefs $ HaskellName f, refName tn c)
+    ref c fe []
+        | ForeignRef f _ <- fe =
+            Just (resolveTableName allDefs f, refName tn c)
         | otherwise = Nothing
     ref _ _ ("noreference":_) = Nothing
     ref c _ (a:_)
@@ -74,13 +70,8 @@ refName :: DBName -> DBName -> DBName
 refName (DBName table) (DBName column) =
     DBName $ mconcat [table, "_", column, "_fkey"]
 
-resolveTableName :: [EntityDef a] -> HaskellName -> DBName
+resolveTableName :: [EntityDef] -> HaskellName -> DBName
 resolveTableName [] (HaskellName hn) = error $ "Table not found: " `mappend` T.unpack hn
 resolveTableName (e:es) hn
     | entityHaskell e == hn = entityDB e
     | otherwise = resolveTableName es hn
-
-convertKey :: Bool -> KeyBackend t t1 -> [PersistValue]
-convertKey True (Key (PersistList fks)) = fks
-convertKey False (Key ret@(PersistInt64 _)) = [ret]
-convertKey composite k = error $ "invalid key type " ++ show k ++ " composite=" ++ show composite
