@@ -4,6 +4,7 @@ module Database.Persist.Sql.Migration
   ( parseMigration
   , parseMigration'
   , printMigration
+  , showMigration
   , getMigration
   , runMigration
   , runMigrationSilent
@@ -16,6 +17,7 @@ import Control.Monad.Trans.Control (MonadBaseControl)
 import Control.Monad.Trans.Class (MonadTrans (..))
 import Control.Monad.IO.Class
 import Control.Monad.Trans.Writer
+import Control.Monad.Trans.Reader (ReaderT (..), ask)
 import Control.Monad (liftM, unless)
 import Data.Text (Text, unpack, snoc, isPrefixOf, pack)
 import qualified Data.Text.IO
@@ -23,77 +25,82 @@ import System.IO
 import System.IO.Silently (hSilence)
 import Control.Monad.Trans.Control (liftBaseOp_)
 import Database.Persist.Sql.Types
-import Database.Persist.Sql.Class
 import Database.Persist.Sql.Raw
 import Database.Persist.Types
 
 allSql :: CautiousMigration -> [Sql]
 allSql = map snd
-unsafeSql :: CautiousMigration -> [Sql]
-unsafeSql = allSql . filter fst
 safeSql :: CautiousMigration -> [Sql]
 safeSql = allSql . filter (not . fst)
 
-parseMigration :: Monad m => Migration m -> m (Either [Text] CautiousMigration)
+parseMigration :: MonadIO m => Migration -> ReaderT SqlBackend m (Either [Text] CautiousMigration)
 parseMigration =
-    liftM go . runWriterT . execWriterT
+    liftIOReader . liftM go . runWriterT . execWriterT
   where
     go ([], sql) = Right sql
     go (errs, _) = Left errs
 
+    liftIOReader (ReaderT m) = ReaderT $ liftIO . m
+
 -- like parseMigration, but call error or return the CautiousMigration
-parseMigration' :: Monad m => Migration m -> m (CautiousMigration)
+parseMigration' :: MonadIO m => Migration -> ReaderT SqlBackend m (CautiousMigration)
 parseMigration' m = do
   x <- parseMigration m
   case x of
       Left errs -> error $ unlines $ map unpack errs
       Right sql -> return sql
 
-printMigration :: MonadIO m => Migration m -> m ()
-printMigration m = do
-  mig <- parseMigration' m
-  mapM_ (liftIO . Data.Text.IO.putStrLn . flip snoc ';') (allSql mig)
+printMigration :: MonadIO m => Migration -> ReaderT SqlBackend m ()
+printMigration m = showMigration m
+               >>= mapM_ (liftIO . Data.Text.IO.putStrLn)
 
-getMigration :: (MonadBaseControl IO m, MonadIO m) => Migration m -> m [Sql]
+showMigration :: MonadIO m => Migration -> ReaderT SqlBackend m [Text]
+showMigration m = map (flip snoc ';') `liftM` getMigration m
+
+getMigration :: MonadIO m => Migration -> ReaderT SqlBackend m [Sql]
 getMigration m = do
   mig <- parseMigration' m
   return $ allSql mig
 
-runMigration :: MonadSqlPersist m
-             => Migration m
-             -> m ()
+runMigration :: MonadIO m
+             => Migration
+             -> ReaderT SqlBackend m ()
 runMigration m = runMigration' m False >> return ()
 
 -- | Same as 'runMigration', but returns a list of the SQL commands executed
 -- instead of printing them to stderr.
-runMigrationSilent :: (MonadBaseControl IO m, MonadSqlPersist m)
-                   => Migration m
-                   -> m [Text]
+runMigrationSilent :: (MonadBaseControl IO m, MonadIO m)
+                   => Migration
+                   -> ReaderT SqlBackend m [Text]
 runMigrationSilent m = liftBaseOp_ (hSilence [stderr]) $ runMigration' m True
 
 runMigration'
-    :: MonadSqlPersist m
-    => Migration m
+    :: MonadIO m
+    => Migration
     -> Bool -- ^ is silent?
-    -> m [Text]
+    -> ReaderT SqlBackend m [Text]
 runMigration' m silent = do
     mig <- parseMigration' m
-    case unsafeSql mig of
-        []   -> mapM (executeMigrate silent) $ sortMigrations $ safeSql mig
-        errs -> error $ concat
-            [ "\n\nDatabase migration: manual intervention required.\n"
-            , "The following actions are considered unsafe:\n\n"
-            , unlines $ map (\s -> "    " ++ unpack s ++ ";") $ errs
-            ]
+    if any fst mig
+        then error $ concat
+                 [ "\n\nDatabase migration: manual intervention required.\n"
+                 , "The unsafe actions are prefixed by '***' below:\n\n"
+                 , unlines $ map displayMigration mig
+                 ]
+        else mapM (executeMigrate silent) $ sortMigrations $ safeSql mig
+  where
+    displayMigration :: (Bool, Sql) -> String
+    displayMigration (True,  s) = "*** " ++ unpack s ++ ";"
+    displayMigration (False, s) = "    " ++ unpack s ++ ";"
 
-runMigrationUnsafe :: MonadSqlPersist m
-                   => Migration m
-                   -> m ()
+runMigrationUnsafe :: MonadIO m
+                   => Migration
+                   -> ReaderT SqlBackend m ()
 runMigrationUnsafe m = do
     mig <- parseMigration' m
     mapM_ (executeMigrate False) $ sortMigrations $ allSql mig
 
-executeMigrate :: MonadSqlPersist m => Bool -> Text -> m Text
+executeMigrate :: MonadIO m => Bool -> Text -> ReaderT SqlBackend m Text
 executeMigrate silent s = do
     unless silent $ liftIO $ hPutStrLn stderr $ "Migrating: " ++ unpack s
     rawExecute s []
@@ -109,11 +116,10 @@ sortMigrations x =
     -- choose to have this special sorting applied.
     isCreate t = pack "CREATe " `isPrefixOf` t
 
-migrate :: MonadSqlPersist m
-        => [EntityDef SqlType]
-        -> EntityDef SqlType
-        -> Migration m
+migrate :: [EntityDef]
+        -> EntityDef
+        -> Migration
 migrate allDefs val = do
-    conn <- askSqlConn
+    conn <- lift $ lift ask
     res <- liftIO $ connMigrateSql conn allDefs (getStmtConn conn) val
     either tell (lift . tell) res
