@@ -1,4 +1,5 @@
 {-# LANGUAGE RankNTypes #-}
+{-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 module Database.Persist.Sql.Orphan.PersistQuery
@@ -7,20 +8,19 @@ module Database.Persist.Sql.Orphan.PersistQuery
     , decorateSQLWithLimitOffset
     ) where
 
-import Database.Persist
+import Database.Persist hiding (updateField)
+import Database.Persist.Sql.Util (
+  entityColumnNames, parseEntityValues, isIdField)
 import Database.Persist.Sql.Types
-import Database.Persist.Sql.Class
 import Database.Persist.Sql.Raw
-import Database.Persist.Sql.Orphan.PersistStore ()
-import Database.Persist.Sql.Internal (convertKey)
+import Database.Persist.Sql.Orphan.PersistStore (withRawQuery)
+import Database.Persist.Sql.Util (dbIdColumns)
 import qualified Data.Text as T
 import Data.Text (Text)
 import Data.Monoid (Monoid (..), (<>))
 import Data.Int (Int64)
-import Control.Monad.Logger
 import Control.Monad.IO.Class
-import Control.Monad.Trans.Class
-import Control.Monad.Trans.Resource (MonadResource)
+import Control.Monad.Trans.Reader (ReaderT, ask)
 import Control.Exception (throwIO)
 import qualified Data.Conduit.List as CL
 import Data.Conduit
@@ -29,36 +29,9 @@ import Data.Maybe (isJust)
 import Data.List (transpose, inits, find)
 
 -- orphaned instance for convenience of modularity
-instance (MonadResource m, MonadLogger m) => PersistQuery (SqlPersistT m) where
-    update _ [] = return ()
-    update k upds = do
-        conn <- askSqlConn
-        let go'' n Assign = n <> "=?"
-            go'' n Add = T.concat [n, "=", n, "+?"]
-            go'' n Subtract = T.concat [n, "=", n, "-?"]
-            go'' n Multiply = T.concat [n, "=", n, "*?"]
-            go'' n Divide = T.concat [n, "=", n, "/?"]
-        let go' (x, pu) = go'' (connEscapeName conn x) pu
-        let composite = isJust $ entityPrimary t
-        let wher = case entityPrimary t of
-                Just pdef -> T.intercalate " AND " $ map (\fld -> connEscapeName conn (snd fld) <> "=? ") $ primaryFields pdef
-                Nothing   -> connEscapeName conn (entityID t) <> "=?"
-        let sql = T.concat
-                [ "UPDATE "
-                , connEscapeName conn $ entityDB t
-                , " SET "
-                , T.intercalate "," $ map (go' . go) upds
-                , " WHERE "
-                , wher
-                ]
-        rawExecute sql $
-            map updatePersistValue upds `mappend` (convertKey composite k)
-      where
-        t = entityDef $ dummyFromKey k
-        go x = (fieldDB $ updateFieldDef x, updateUpdate x)
-
+instance PersistQuery SqlBackend where
     count filts = do
-        conn <- askSqlConn
+        conn <- ask
         let wher = if null filts
                     then ""
                     else filterClause False conn filts
@@ -67,7 +40,7 @@ instance (MonadResource m, MonadLogger m) => PersistQuery (SqlPersistT m) where
                 , connEscapeName conn $ entityDB t
                 , wher
                 ]
-        rawQuery sql (getFiltsValues conn filts) $$ do
+        withRawQuery sql (getFiltsValues conn filts) $ do
             mm <- CL.head
             case mm of
               Just [PersistInt64 i] -> return $ fromIntegral i
@@ -80,42 +53,17 @@ instance (MonadResource m, MonadLogger m) => PersistQuery (SqlPersistT m) where
       where
         t = entityDef $ dummyFromFilts filts
 
-    selectSource filts opts = do
-        conn <- lift askSqlConn
-        rawQuery (sql conn) (getFiltsValues conn filts) $= CL.mapM parse
+    selectSourceRes filts opts = do
+        conn <- ask
+        srcRes <- rawQueryRes (sql conn) (getFiltsValues conn filts)
+        return $ fmap ($= CL.mapM parse) srcRes
       where
-        composite = isJust $ entityPrimary t
         (limit, offset, orders) = limitOffsetOrder opts
 
-        parse vals =
-          case entityPrimary t of
-            Just pdef -> 
-                  let pks = map fst $ primaryFields pdef
-                      keyvals = map snd $ filter (\(a, _) -> let ret=isJust (find (== a) pks) in ret) $ zip (map fieldHaskell $ entityFields t) vals
-                  in case fromPersistValuesComposite' keyvals vals of
-                      Left s -> liftIO $ throwIO $ PersistMarshalError s
-                      Right row -> return row
-            Nothing -> 
-                  case fromPersistValues' vals of
-                    Left s -> liftIO $ throwIO $ PersistMarshalError s
-                    Right row -> return row
-
+        parse vals = case parseEntityValues t vals of
+                       Left s -> liftIO $ throwIO $ PersistMarshalError s
+                       Right row -> return row
         t = entityDef $ dummyFromFilts filts
-        fromPersistValues' (PersistInt64 x:xs) = 
-            case fromPersistValues xs of
-                Left e -> Left e
-                Right xs' -> Right (Entity (Key $ PersistInt64 x) xs')
-        fromPersistValues' (PersistDouble x:xs) = -- oracle returns Double 
-            case fromPersistValues xs of
-                Left e -> Left e
-                Right xs' -> Right (Entity (Key $ PersistInt64 (truncate x)) xs') -- convert back to int64
-        fromPersistValues' xs = Left $ T.pack ("error in fromPersistValues' xs=" ++ show xs)
-
-        fromPersistValuesComposite' keyvals xs =
-            case fromPersistValues xs of
-                Left e -> Left e
-                Right xs' -> Right (Entity (Key $ PersistList keyvals) xs')
-
         wher conn = if null filts
                     then ""
                     else filterClause False conn filts
@@ -123,9 +71,7 @@ instance (MonadResource m, MonadLogger m) => PersistQuery (SqlPersistT m) where
             case map (orderClause False conn) orders of
                 [] -> ""
                 ords -> " ORDER BY " <> T.intercalate "," ords
-        cols conn = T.intercalate ","
-                  $ ((if composite then [] else [connEscapeName conn $ entityID t]) 
-                  <> map (connEscapeName conn . fieldDB) (entityFields t))
+        cols = T.intercalate ", " . entityColumnNames t
         sql conn = connLimitOffset conn (limit,offset) (not (null orders)) $ mconcat
             [ "SELECT "
             , cols conn
@@ -135,15 +81,15 @@ instance (MonadResource m, MonadLogger m) => PersistQuery (SqlPersistT m) where
             , ord conn
             ]
 
-    selectKeys filts opts = do
-        conn <- lift askSqlConn
-        rawQuery (sql conn) (getFiltsValues conn filts) $= CL.mapM parse
+    selectKeysRes filts opts = do
+        conn <- ask
+        srcRes <- rawQueryRes (sql conn) (getFiltsValues conn filts)
+        return $ fmap ($= CL.mapM parse) srcRes
       where
         t = entityDef $ dummyFromFilts filts
-        cols conn = case entityPrimary t of 
-                     Just pdef -> T.intercalate "," $ map (connEscapeName conn . snd) $ primaryFields pdef
-                     Nothing   -> connEscapeName conn $ entityID t
+        cols conn = T.intercalate "," $ dbIdColumns conn t
                       
+
         wher conn = if null filts
                     then ""
                     else filterClause False conn filts
@@ -163,16 +109,22 @@ instance (MonadResource m, MonadLogger m) => PersistQuery (SqlPersistT m) where
                 [] -> ""
                 ords -> " ORDER BY " <> T.intercalate "," ords
 
-        parse xs = case entityPrimary t of
+        parse xs = do
+            keyvals <- case entityPrimary t of
                       Nothing -> 
                         case xs of
-                           [PersistInt64 x] -> return $ Key $ PersistInt64 x
-                           [PersistDouble x] -> return $ Key $ PersistInt64 (truncate x) -- oracle returns Double 
+                           [PersistInt64 x] -> return [PersistInt64 x]
+                           [PersistDouble x] -> return [PersistInt64 (truncate x)] -- oracle returns Double 
                            _ -> liftIO $ throwIO $ PersistMarshalError $ "Unexpected in selectKeys False: " <> T.pack (show xs)
                       Just pdef -> 
-                           let pks = map fst $ primaryFields pdef
+                           let pks = map fieldHaskell $ compositeFields pdef
                                keyvals = map snd $ filter (\(a, _) -> let ret=isJust (find (== a) pks) in ret) $ zip (map fieldHaskell $ entityFields t) xs
-                           in return $ Key $ PersistList keyvals
+                           in return keyvals
+            case keyFromValues keyvals of
+                Right k -> return k
+                Left _ -> error "selectKeysImpl: keyFromValues failed"
+
+
 
     deleteWhere filts = do
         _ <- deleteWhereCount filts
@@ -185,11 +137,11 @@ instance (MonadResource m, MonadLogger m) => PersistQuery (SqlPersistT m) where
 -- | Same as 'deleteWhere', but returns the number of rows affected.
 --
 -- Since 1.1.5
-deleteWhereCount :: (PersistEntity val, MonadSqlPersist m)
+deleteWhereCount :: (PersistEntity val, MonadIO m, PersistEntityBackend val ~ SqlBackend)
                  => [Filter val]
-                 -> m Int64
+                 -> ReaderT SqlBackend m Int64
 deleteWhereCount filts = do
-    conn <- askSqlConn
+    conn <- ask
     let t = entityDef $ dummyFromFilts filts
     let wher = if null filts
                 then ""
@@ -204,13 +156,13 @@ deleteWhereCount filts = do
 -- | Same as 'updateWhere', but returns the number of rows affected.
 --
 -- Since 1.1.5
-updateWhereCount :: (PersistEntity val, MonadSqlPersist m)
+updateWhereCount :: (PersistEntity val, MonadIO m, SqlBackend ~ PersistEntityBackend val)
                  => [Filter val]
                  -> [Update val]
-                 -> m Int64
+                 -> ReaderT SqlBackend m Int64
 updateWhereCount _ [] = return 0
 updateWhereCount filts upds = do
-    conn <- askSqlConn
+    conn <- ask
     let wher = if null filts
                 then ""
                 else filterClause False conn filts
@@ -231,24 +183,29 @@ updateWhereCount filts upds = do
     go'' n Subtract = mconcat [n, "=", n, "-?"]
     go'' n Multiply = mconcat [n, "=", n, "*?"]
     go'' n Divide = mconcat [n, "=", n, "/?"]
+    go'' _ (BackendSpecificUpdate up) = error $ T.unpack $ "BackendSpecificUpdate" `mappend` up `mappend` "not supported"
     go' conn (x, pu) = go'' (connEscapeName conn x) pu
-    go x = (fieldDB $ updateFieldDef x, updateUpdate x)
+    go x = (updateField x, updateUpdate x)
 
-updateFieldDef :: PersistEntity v => Update v -> FieldDef SqlType
-updateFieldDef (Update f _ _) = persistFieldDef f
+    updateField (Update f _ _) = fieldName f
+    updateField _ = error "BackendUpdate not implemented"
+
+fieldName ::  forall record typ.  (PersistEntity record, PersistEntityBackend record ~ SqlBackend) => EntityField record typ -> DBName
+fieldName f = fieldDB $ persistFieldDef f
 
 dummyFromFilts :: [Filter v] -> Maybe v
 dummyFromFilts _ = Nothing
 
-getFiltsValues :: forall val.  PersistEntity val => Connection -> [Filter val] -> [PersistValue]
+getFiltsValues :: forall val. (PersistEntity val, PersistEntityBackend val ~ SqlBackend)
+               => SqlBackend -> [Filter val] -> [PersistValue]
 getFiltsValues conn = snd . filterClauseHelper False False conn OrNullNo
 
 data OrNull = OrNullYes | OrNullNo
 
-filterClauseHelper :: PersistEntity val
+filterClauseHelper :: (PersistEntity val, PersistEntityBackend val ~ SqlBackend)
              => Bool -- ^ include table name?
              -> Bool -- ^ include WHERE?
-             -> Connection
+             -> SqlBackend
              -> OrNull
              -> [Filter val]
              -> (Text, [PersistValue])
@@ -273,31 +230,30 @@ filterClauseHelper includeTable includeWhere conn orNull filters =
     go (FilterOr fs)  = combine " OR " fs
     go (Filter field value pfilter) = 
         let t = entityDef $ dummyFromFilts [Filter field value pfilter]
-        in case (fieldDB (persistFieldDef field) == DBName "id", entityPrimary t, allVals) of
-            -- need to check the id field in a safer way: entityId? 
-                 (True, Just pdef, (PersistList ys:_)) -> 
-                    if length (primaryFields pdef) /= length ys 
-                       then error $ "wrong number of entries in primaryFields vs PersistList allVals=" ++ show allVals
+        in case (isIdField field, entityPrimary t, allVals) of
+                 (True, Just pdef, PersistList ys:_) ->
+                    if length (compositeFields pdef) /= length ys
+                       then error $ "wrong number of entries in compositeFields vs PersistList allVals=" ++ show allVals
                     else
                       case (allVals, pfilter, isCompFilter pfilter) of
                         ([PersistList xs], Eq, _) -> 
-                           let sqlcl=T.intercalate " and " (map (\a -> connEscapeName conn (snd a) <> showSqlFilter pfilter <> "? ")  (primaryFields pdef))
+                           let sqlcl=T.intercalate " and " (map (\a -> connEscapeName conn (fieldDB a) <> showSqlFilter pfilter <> "? ")  (compositeFields pdef))
                            in (wrapSql sqlcl,xs)
                         ([PersistList xs], Ne, _) -> 
-                           let sqlcl=T.intercalate " or " (map (\a -> connEscapeName conn (snd a) <> showSqlFilter pfilter <> "? ")  (primaryFields pdef))
+                           let sqlcl=T.intercalate " or " (map (\a -> connEscapeName conn (fieldDB a) <> showSqlFilter pfilter <> "? ")  (compositeFields pdef))
                            in (wrapSql sqlcl,xs)
                         (_, In, _) -> 
                            let xxs = transpose (map fromPersistList allVals)
-                               sqls=map (\(a,xs) -> connEscapeName conn (snd a) <> showSqlFilter pfilter <> "(" <> T.intercalate "," (replicate (length xs) " ?") <> ") ") (zip (primaryFields pdef) xxs)
+                               sqls=map (\(a,xs) -> connEscapeName conn (fieldDB a) <> showSqlFilter pfilter <> "(" <> T.intercalate "," (replicate (length xs) " ?") <> ") ") (zip (compositeFields pdef) xxs)
                            in (wrapSql (T.intercalate " and " (map wrapSql sqls)), concat xxs)
                         (_, NotIn, _) -> 
                            let xxs = transpose (map fromPersistList allVals)
-                               sqls=map (\(a,xs) -> connEscapeName conn (snd a) <> showSqlFilter pfilter <> "(" <> T.intercalate "," (replicate (length xs) " ?") <> ") ") (zip (primaryFields pdef) xxs)
+                               sqls=map (\(a,xs) -> connEscapeName conn (fieldDB a) <> showSqlFilter pfilter <> "(" <> T.intercalate "," (replicate (length xs) " ?") <> ") ") (zip (compositeFields pdef) xxs)
                            in (wrapSql (T.intercalate " or " (map wrapSql sqls)), concat xxs)
                         ([PersistList xs], _, True) -> 
-                           let zs = tail (inits (primaryFields pdef))
+                           let zs = tail (inits (compositeFields pdef))
                                sql1 = map (\b -> wrapSql (T.intercalate " and " (map (\(i,a) -> sql2 (i==length b) a) (zip [1..] b)))) zs
-                               sql2 islast a = connEscapeName conn (snd a) <> (if islast then showSqlFilter pfilter else showSqlFilter Eq) <> "? "
+                               sql2 islast a = connEscapeName conn (fieldDB a) <> (if islast then showSqlFilter pfilter else showSqlFilter Eq) <> "? "
                                sqlcl = T.intercalate " or " sql1
                            in (wrapSql sqlcl, concat (tail (inits xs)))
                         (_, BackendSpecificFilter _, _) -> error "unhandled type BackendSpecificFilter for composite/non id primary keys"
@@ -378,7 +334,7 @@ filterClauseHelper includeTable includeWhere conn orNull filters =
             (if includeTable
                 then ((tn <> ".") <>)
                 else id)
-            $ connEscapeName conn $ fieldDB $ persistFieldDef field
+            $ connEscapeName conn $ fieldName field
         qmarks = case value of
                     Left _ -> "?"
                     Right x ->
@@ -399,38 +355,38 @@ filterClauseHelper includeTable includeWhere conn orNull filters =
 
 updatePersistValue :: Update v -> PersistValue
 updatePersistValue (Update _ v _) = toPersistValue v
+updatePersistValue _ = error "BackendUpdate not implemented"
 
-filterClause :: PersistEntity val
+filterClause :: (PersistEntity val, PersistEntityBackend val ~ SqlBackend)
              => Bool -- ^ include table name?
-             -> Connection
+             -> SqlBackend
              -> [Filter val]
              -> Text
 filterClause b c = fst . filterClauseHelper b True c OrNullNo
 
-orderClause :: PersistEntity val
+orderClause :: (PersistEntity val, PersistEntityBackend val ~ SqlBackend)
             => Bool -- ^ include the table name
-            -> Connection
+            -> SqlBackend
             -> SelectOpt val
             -> Text
 orderClause includeTable conn o =
     case o of
-        Asc  x -> name $ persistFieldDef x
-        Desc x -> name (persistFieldDef x) <> " DESC"
-        _ -> error $ "orderClause: expected Asc or Desc, not limit or offset"
+        Asc  x -> name x
+        Desc x -> name x <> " DESC"
+        _ -> error "orderClause: expected Asc or Desc, not limit or offset"
   where
     dummyFromOrder :: SelectOpt a -> Maybe a
     dummyFromOrder _ = Nothing
 
     tn = connEscapeName conn $ entityDB $ entityDef $ dummyFromOrder o
 
+    name :: (PersistEntityBackend record ~ SqlBackend, PersistEntity record)
+         => EntityField record typ -> Text
     name x =
         (if includeTable
             then ((tn <> ".") <>)
             else id)
-        $ connEscapeName conn $ fieldDB x
-
-dummyFromKey :: KeyBackend SqlBackend v -> Maybe v
-dummyFromKey _ = Nothing
+        $ connEscapeName conn $ fieldName x
 
 -- | Generates sql for limit and offset for postgres, sqlite and mysql.
 decorateSQLWithLimitOffset::Text -> (Int,Int) -> Bool -> Text -> Text 
