@@ -20,6 +20,7 @@ module Database.Persist.Sql.Orphan.PersistStore
 import Database.Persist
 import Database.Persist.Sql.Types
 import Database.Persist.Sql.Raw
+import Database.Persist.Sql.Util (dbIdColumns, keyAndEntityColumnNames)
 import qualified Data.Conduit as C
 import qualified Data.Conduit.List as CL
 import qualified Data.Text as T
@@ -33,6 +34,7 @@ import Control.Monad.Trans.Reader (ReaderT, ask)
 import Data.Acquire (with)
 import Data.Int (Int64)
 import Web.PathPieces (PathPiece)
+import Web.HttpApiData (ToHttpApiData, FromHttpApiData)
 import Database.Persist.Sql.Class (PersistFieldSql)
 import qualified Data.Aeson as A
 import Control.Exception.Lifted (throwIO)
@@ -54,10 +56,11 @@ fromSqlKey = unSqlBackendKey . toBackendKey
 
 whereStmtForKey :: PersistEntity record => SqlBackend -> Key record -> Text
 whereStmtForKey conn k =
-  case entityPrimary t of
-    Just pdef -> T.intercalate " AND " $ map (\fld -> connEscapeName conn (fieldDB fld) <> "=? ") $ compositeFields pdef
-    Nothing   -> connEscapeName conn (fieldDB (entityId t)) <> "=?"
-  where t = entityDef $ dummyFromKey k
+    T.intercalate " AND "
+  $ map (<> "=? ")
+  $ dbIdColumns conn entDef
+  where
+    entDef = entityDef $ dummyFromKey k
 
 
 -- | get the SQL string for the table that a PeristEntity represents
@@ -103,7 +106,7 @@ fieldDBName = fieldDB . persistFieldDef
 
 instance PersistStore SqlBackend where
     newtype BackendKey SqlBackend = SqlBackendKey { unSqlBackendKey :: Int64 }
-        deriving (Show, Read, Eq, Ord, Num, Integral, PersistField, PersistFieldSql, PathPiece, Real, Enum, Bounded, A.ToJSON, A.FromJSON)
+        deriving (Show, Read, Eq, Ord, Num, Integral, PersistField, PersistFieldSql, PathPiece, ToHttpApiData, FromHttpApiData, Real, Enum, Bounded, A.ToJSON, A.FromJSON)
 
     update _ [] = return ()
     update k upds = do
@@ -169,10 +172,10 @@ instance PersistStore SqlBackend where
                             Right k -> return k
                             Left err -> throw $ "ISRInsertGet: keyFromValues failed: " `mappend` err
                 ISRManyKeys sql fs -> do
-                    rawExecute sql vals 
+                    rawExecute sql vals
                     case entityPrimary t of
                        Nothing -> error $ "ISRManyKeys is used when Primary is defined " ++ show sql
-                       Just pdef -> 
+                       Just pdef ->
                             let pks = map fieldHaskell $ compositeFields pdef
                                 keyvals = map snd $ filter (\(a, _) -> let ret=isJust (find (== a) pks) in ret) $ zip (map fieldHaskell $ entityFields t) fs
                             in  case keyFromValues keyvals of
@@ -187,6 +190,21 @@ instance PersistStore SqlBackend where
         t = entityDef $ Just val
         vals = map toPersistValue $ toPersistFields val
 
+    insertMany [] = return []
+    insertMany vals = do
+        conn <- ask
+
+        case connInsertManySql conn of
+            Nothing -> mapM insert vals
+            Just insertManyFn -> do
+                case insertManyFn ent valss of
+                    ISRSingle sql -> rawSql sql (concat valss)
+                    _ -> error "ISRSingle is expected from the connInsertManySql function"
+                where
+                    ent = entityDef vals
+                    valss = map (map toPersistValue . toPersistFields) vals
+
+
     insertMany_ [] = return ()
     insertMany_ vals = do
         conn <- ask
@@ -199,11 +217,7 @@ instance PersistStore SqlBackend where
                 , T.intercalate "),(" $ replicate (length valss) $ T.intercalate "," $ map (const "?") (entityFields t)
                 , ")"
                 ]
-
-        -- SQLite support is only in later versions
-        if connRDBMS conn == "sqlite"
-            then mapM_ insert vals
-            else rawExecute sql (concat valss)
+        rawExecute sql (concat valss)
       where
         t = entityDef vals
         valss = map (map toPersistValue . toPersistFields) vals
@@ -281,24 +295,23 @@ insrepHelper :: (MonadIO m, PersistEntity val)
              -> Key val
              -> val
              -> ReaderT SqlBackend m ()
-insrepHelper command k val = do
+insrepHelper command k record = do
     conn <- ask
-    rawExecute (sql conn) vals
+    let columnNames = keyAndEntityColumnNames entDef conn
+    rawExecute (sql conn columnNames) vals
   where
-    t = entityDef $ Just val
-    sql conn = T.concat
+    entDef = entityDef $ Just record
+    sql conn columnNames = T.concat
         [ command
         , " INTO "
-        , connEscapeName conn (entityDB t)
+        , connEscapeName conn (entityDB entDef)
         , "("
-        , T.intercalate ","
-            $ map (connEscapeName conn)
-            $ fieldDB (entityId t) : map fieldDB (entityFields t)
+        , T.intercalate "," columnNames
         , ") VALUES("
-        , T.intercalate "," ("?" : map (const "?") (entityFields t))
+        , T.intercalate "," (map (const "?") columnNames)
         , ")"
         ]
-    vals = keyToValues k ++ map toPersistValue (toPersistFields val)
+    vals = entityValues (Entity k record)
 
 updateFieldDef :: PersistEntity v => Update v -> FieldDef
 updateFieldDef (Update f _ _) = persistFieldDef f
