@@ -1,52 +1,77 @@
-{-# LANGUAGE TemplateHaskell #-}
 {-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE FlexibleContexts #-}
 module Database.Persist.Sql.Raw where
 
 import Database.Persist
 import Database.Persist.Sql.Types
 import Database.Persist.Sql.Class
 import qualified Data.Map as Map
-import Control.Monad.IO.Class (liftIO)
+import Control.Monad.IO.Class (MonadIO, liftIO)
+import Control.Monad.Reader (ReaderT, ask, MonadReader)
+import Data.Acquire (allocateAcquire, Acquire, mkAcquire, with)
 import Data.IORef (writeIORef, readIORef, newIORef)
 import Control.Exception (throwIO)
 import Control.Monad (when, liftM)
 import Data.Text (Text, pack)
-import Control.Monad.Logger (logDebugS)
+import Control.Monad.Logger (logDebugNS, runLoggingT)
 import Data.Int (Int64)
-import Control.Monad.Trans.Class (lift)
 import qualified Data.Text as T
 import Data.Conduit
-import Control.Monad.Trans.Resource (MonadResource)
+import Control.Monad.Trans.Resource (MonadResource,release)
 
-rawQuery :: (MonadSqlPersist m, MonadResource m)
+rawQuery :: (MonadResource m, MonadReader env m, HasPersistBackend env SqlBackend)
          => Text
          -> [PersistValue]
          -> Source m [PersistValue]
 rawQuery sql vals = do
-    lift $ $logDebugS (pack "SQL") $ pack $ show sql ++ " " ++ show vals
-    conn <- lift askSqlConn
-    bracketP
-        (getStmtConn conn sql)
-        stmtReset
-        (flip stmtQuery vals)
+    srcRes <- liftPersist $ rawQueryRes sql vals
+    (releaseKey, src) <- allocateAcquire srcRes
+    src
+    release releaseKey
 
-rawExecute :: MonadSqlPersist m => Text -> [PersistValue] -> m ()
+rawQueryRes
+    :: (MonadIO m1, MonadIO m2)
+    => Text
+    -> [PersistValue]
+    -> ReaderT SqlBackend m1 (Acquire (Source m2 [PersistValue]))
+rawQueryRes sql vals = do
+    conn <- ask
+    let make = do
+            runLoggingT (logDebugNS (pack "SQL") $ T.append sql $ pack $ "; " ++ show vals)
+                (connLogFunc conn)
+            getStmtConn conn sql
+    return $ do
+        stmt <- mkAcquire make stmtReset
+        stmtQuery stmt vals
+
+-- | Execute a raw SQL statement
+rawExecute :: MonadIO m
+           => Text            -- ^ SQL statement, possibly with placeholders.
+           -> [PersistValue]  -- ^ Values to fill the placeholders.
+           -> ReaderT SqlBackend m ()
 rawExecute x y = liftM (const ()) $ rawExecuteCount x y
 
-rawExecuteCount :: MonadSqlPersist m => Text -> [PersistValue] -> m Int64
+-- | Execute a raw SQL statement and return the number of
+-- rows it has modified.
+rawExecuteCount :: MonadIO m
+                => Text            -- ^ SQL statement, possibly with placeholders.
+                -> [PersistValue]  -- ^ Values to fill the placeholders.
+                -> ReaderT SqlBackend m Int64
 rawExecuteCount sql vals = do
-    $logDebugS (pack "SQL") $ pack $ show sql ++ " " ++ show vals
+    conn <- ask
+    runLoggingT (logDebugNS (pack "SQL") $ T.append sql $ pack $ "; " ++ show vals)
+        (connLogFunc conn)
     stmt <- getStmt sql
     res <- liftIO $ stmtExecute stmt vals
     liftIO $ stmtReset stmt
     return res
 
-getStmt :: MonadSqlPersist m => Text -> m Statement
+getStmt :: MonadIO m => Text -> ReaderT SqlBackend m Statement
 getStmt sql = do
-    conn <- askSqlConn
+    conn <- ask
     liftIO $ getStmtConn conn sql
 
-getStmtConn :: Connection -> Text -> IO Statement
+getStmtConn :: SqlBackend -> Text -> IO Statement
 getStmtConn conn sql = do
     smap <- liftIO $ readIORef $ connStmtMap conn
     case Map.lookup sql smap of
@@ -101,10 +126,84 @@ getStmtConn conn sql = do
 -- However, most common problems are mitigated by using the
 -- entity selection placeholder @??@, and you shouldn't see any
 -- error at all if you're not using 'Single'.
-rawSql :: (RawSql a, MonadSqlPersist m, MonadResource m)
+--
+-- Some example of 'rawSql' based on this schema:
+--
+-- @
+-- share [mkPersist sqlSettings, mkMigrate "migrateAll"] [persistLowerCase|
+-- Person
+--     name String
+--     age Int Maybe
+--     deriving Show
+-- BlogPost
+--     title String
+--     authorId PersonId
+--     deriving Show
+-- |]
+-- @
+-- 
+-- Examples based on the above schema:
+-- 
+-- @ 
+-- getPerson :: MonadIO m => ReaderT SqlBackend m [Entity Person]
+-- getPerson = rawSql "select ?? from person where name=?" [PersistText "john"]
+-- 
+-- getAge :: MonadIO m => ReaderT SqlBackend m [Single Int]
+-- getAge = rawSql "select person.age from person where name=?" [PersistText "john"]
+-- 
+-- getAgeName :: MonadIO m => ReaderT SqlBackend m [(Single Int, Single Text)]
+-- getAgeName = rawSql "select person.age, person.name from person where name=?" [PersistText "john"]
+-- 
+-- getPersonBlog :: MonadIO m => ReaderT SqlBackend m [(Entity Person, Entity BlogPost)]
+-- getPersonBlog = rawSql "select ??,?? from person,blog_post where person.id = blog_post.author_id" []
+-- @
+--
+-- Minimal working program for PostgreSQL backend based on the above concepts:
+--
+-- > {-# LANGUAGE EmptyDataDecls             #-}
+-- > {-# LANGUAGE FlexibleContexts           #-}
+-- > {-# LANGUAGE GADTs                      #-}
+-- > {-# LANGUAGE GeneralizedNewtypeDeriving #-}
+-- > {-# LANGUAGE MultiParamTypeClasses      #-}
+-- > {-# LANGUAGE OverloadedStrings          #-}
+-- > {-# LANGUAGE QuasiQuotes                #-}
+-- > {-# LANGUAGE TemplateHaskell            #-}
+-- > {-# LANGUAGE TypeFamilies               #-}
+-- > 
+-- > import           Control.Monad.IO.Class  (liftIO)
+-- > import           Control.Monad.Logger    (runStderrLoggingT)
+-- > import           Database.Persist
+-- > import           Control.Monad.Reader
+-- > import           Data.Text
+-- > import           Database.Persist.Sql
+-- > import           Database.Persist.Postgresql
+-- > import           Database.Persist.TH
+-- > 
+-- > share [mkPersist sqlSettings, mkMigrate "migrateAll"] [persistLowerCase|
+-- > Person
+-- >     name String
+-- >     age Int Maybe
+-- >     deriving Show
+-- > |]
+-- > 
+-- > conn = "host=localhost dbname=new_db user=postgres password=postgres port=5432"
+-- > 
+-- > getPerson :: MonadIO m => ReaderT SqlBackend m [Entity Person]
+-- > getPerson = rawSql "select ?? from person where name=?" [PersistText "sibi"]
+-- > 
+-- > liftSqlPersistMPool y x = liftIO (runSqlPersistMPool y x)
+-- > 
+-- > main :: IO ()
+-- > main = runStderrLoggingT $ withPostgresqlPool conn 10 $ liftSqlPersistMPool $ do
+-- >          runMigration migrateAll
+-- >          xs <- getPerson
+-- >          liftIO (print xs)
+-- > 
+
+rawSql :: (RawSql a, MonadIO m)
        => Text             -- ^ SQL statement, possibly with placeholders.
        -> [PersistValue]   -- ^ Values to fill the placeholders.
-       -> m [a]
+       -> ReaderT SqlBackend m [a]
 rawSql stmt = run
     where
       getType :: (x -> m [a]) -> a
@@ -113,8 +212,9 @@ rawSql stmt = run
       x = getType run
       process = rawSqlProcessRow
 
-      withStmt' colSubsts params = do
-            rawQuery sql params
+      withStmt' colSubsts params sink = do
+            srcRes <- rawQueryRes sql params
+            liftIO $ with srcRes ($$ sink)
           where
             sql = T.concat $ makeSubsts colSubsts $ T.splitOn placeholder stmt
             placeholder = "??"
@@ -131,9 +231,9 @@ rawSql stmt = run
                         ]
 
       run params = do
-        conn <- askSqlConn
+        conn <- ask
         let (colCount, colSubsts) = rawSqlCols (connEscapeName conn) x
-        withStmt' colSubsts params $$ firstRow colCount
+        withStmt' colSubsts params $ firstRow colCount
 
       firstRow colCount = do
         mrow <- await
